/***************************************************************
 *
 * Copyright (C) 1990-2007, Condor Team, Computer Sciences Department,
 * University of Wisconsin-Madison, WI.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you
 * may not use this file except in compliance with the License.  You may
 * obtain a copy of the License at
 * 
 *    http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ***************************************************************/

#define _CONDOR_ALLOW_OPEN 1
#include "condor_common.h"
#include "condor_nfs.h"
#include "condor_classad.h"
#include "condor_debug.h"
#include "std_univ_io.h"
#include "condor_sys.h"
#include "condor_mmap.h"
#include "startup.h"
#include "pseudo_ops.h"
#include "../condor_shadow.std/job_report.h"

	/*
	Have assert() really call EXCEPT instead of doing a sigabrt 
	and dumping core.  This change results in a shadow exception when
	communication with the remote job is interrupted in the middle of 
	a remote system call, instead of the shadow dumping core in the
	user's current working directory.
	Note: the definition to CONDOR_ASSERT() macro is in condor_debug.h
	-Todd <tannenba@cs.wisc.edu> 10/10/2000
	*/
#ifdef assert
#	undef assert
#endif
#define assert(cond) CONDOR_ASSERT(cond)

/* get rid of the access remap to access_euid for the syscall library */
#ifdef access
#	undef access
#endif

#if defined(Solaris)
extern "C" {
	int flock( int, int );
}
#endif

#if defined(HPUX)
extern "C" {
	int flock( int, int );
}
#endif

#if defined(LINUX)
extern "C" {
	
	ssize_t getdirentries(int fd, char *buf, size_t nbytes, off_t *basep);
	ssize_t getdirentries64(int fd, char *buf, size_t nbytes, off64_t *basep);

	/*
	There is no public interface to getdents in the Linux C library.
	So, here we provide one based on a better-known public function.
	*/

	/* It angers me that this is here because basically it creates a definition
		for getdents when linking with the standard universe shadow.
		-psilord */

	static int getdents( int fd, struct dirent *buf, size_t nbytes ) {
		off_t basep;
		basep = lseek(fd,0,SEEK_CUR);
		return getdirentries(fd,(char*)buf,nbytes,&basep);
	}

	static int getdents64( int fd, struct dirent64 *buf, size_t nbytes ) {
		off64_t basep;
		basep = lseek64(fd,0,SEEK_CUR);
		return getdirentries64(fd,(char*)buf,nbytes,&basep);
	}
}
#endif

#if defined(Solaris) || defined(LINUX)
/* there doesn't seem to be a correct prototype of this in the header files.. */
extern "C" {
int getdents64( int fd, struct dirent64 *buf, size_t nbytes );
}
#endif

#include "../condor_syscall_lib/syscall_param_sizes.h"

extern StdUnivSock *syscall_sock;

#if defined(SYSCALL_DEBUG)
	extern char *SyscallLabel;
#endif

int CONDOR_NotSupported( int num, ... );
int CONDOR_Ignored( int num, ... );

#if defined(USE_STUB_CLUMPS)
int do_REMOTE_syscall1(int);
int do_REMOTE_syscall2(int);
int do_REMOTE_syscall3(int);
int do_REMOTE_syscall4(int);
int do_REMOTE_syscall5(int);
#endif

#if !defined(PVM_RECEIVE)
int
do_REMOTE_syscall()
{
	int condor_sysnum;
	int	rval;

	syscall_sock->decode();

	rval = syscall_sock->code(condor_sysnum);
	if(!rval) {
		const char *error;
		if(errno==0) {
			error = "disconnected";
		} else {
			error = strerror(errno);
		}
		EXCEPT("Unable to talk to job: %s",error);
	}

#if defined(SYSCALL_DEBUG)
	sprintf( SyscallLabel, "%-6d", condor_sysnum );
#endif

	dprintf(D_SYSCALLS,
		"Got request for syscall %d <%s>\n",
		condor_sysnum, _condor_syscall_name(condor_sysnum)
	);

	job_report_store_call( condor_sysnum );

	switch( condor_sysnum ) {
#else

#include "joblist.h"
extern struct tagJobList	*JobList;

#include "prototypes.h"

int
do_REMOTE_syscall(StdUnivSock *sock, int process)
{
	int condor_sysnum;
	int rval;
	int	read_ok;

	syscall_sock->decode();

	read_ok = syscall_sock->code(condor_sysnum);

	if( !read_ok ) {
		dprintf( D_ALWAYS, 
		"Failed to get syscall_code for proc %d removing..\n", process);

		RemoveProc( process );
		return -1;
	}

	dprintf( D_SYSCALLS, "Shadow: got request for syscall %d <%s>\n",
			condor_sysnum, _condor_syscall_name(condor_sysnum) );

	switch( condor_sysnum ) {
        case CONDOR_work_request:
          {
                ClassAd *ad = NULL;
                char *a_out;
                char *targ;
                char *orig;
                condor_signal_t *kill_sig;
                condor_errno_t terrno;

                kill_sig = (condor_signal_t *)malloc( (unsigned)INT_SIZE );
                orig = NULL;
                targ = NULL;
                a_out = NULL;
		/* space for ad allocated by pseudo_work_req */
                assert( syscall_sock->end_of_message() );;

                errno = 0;
                rval = pseudo_work_request( ad, a_out, targ, orig, (int *)kill_sig );
                terrno = (condor_errno_t)errno;
                dprintf( D_SYSCALLS, "\trval = %d, errno = %d\n", rval, (int)terrno);

                syscall_sock->encode();
                assert( syscall_sock->code(rval) );
                if( rval < 0 ) {
                        assert( syscall_sock->code(terrno) );
                }
                if( rval >= 0 ) {
                        assert( syscall_sock->code(kill_sig) );
                        assert( syscall_sock->code(orig) );
                        assert( syscall_sock->code(targ) );
                        assert( syscall_sock->code(a_out) );
			assert( ad->put(*syscall_sock) );
                }
                free( (char *)kill_sig );
                free( (char *)orig );
                free( (char *)targ );
                free( (char *)a_out );
		if ( ad != NULL ) delete ad;
                assert( syscall_sock->end_of_message() );;
                return 0;
        }
#endif  // of ifdef PVM_RECEIVE
	case CONDOR_startup_info_request:
	  {
		STARTUP_INFO *s;
		condor_errno_t terrno;

		s = (STARTUP_INFO *)malloc( (unsigned)STARTUP_INFO_SIZE );
		memset( s, 0, (unsigned)STARTUP_INFO_SIZE );
		assert( syscall_sock->end_of_message() );;

		errno = 0;
		rval = pseudo_startup_info_request( s );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\trval = %d, errno = %d\n", rval, (int)terrno );
		syscall_sock->encode();
		assert( syscall_sock->code(rval) );
		if( rval < 0 ) {
			assert( syscall_sock->code(terrno) );
		}
		if( rval >= 0 ) {
			assert( syscall_sock->code(s) );
		}
		free( s->cmd );
		free( s->args_v1or2 );
		free( s->env_v1or2 );
		free( s->iwd );
		free( s );

		assert( syscall_sock->end_of_message() );;
		return 0;
      }
#if defined(LINUX)
	case CONDOR_mmap:
	  {
        // Copied from generated code...
        void *  ret_val;
        void *  a;
        size_t   l;
        int   p;
        int   f;
        int   fd;
        off_t   o;
        condor_errno_t terrno;

        assert( syscall_sock->code(l) );
        dprintf( D_SYSCALLS, "  l = %lu\n", l );
        assert( syscall_sock->code(p) );
        dprintf( D_SYSCALLS, "  p = %d\n", p );
        assert( syscall_sock->code(f) );
        dprintf( D_SYSCALLS, "  f = %d\n", f );
        assert( syscall_sock->code(fd) );
        dprintf( D_SYSCALLS, "  fd = %d\n", fd );
        assert( syscall_sock->code(o) );
        dprintf( D_SYSCALLS, "  o = %lu\n", o );
        assert( syscall_sock->end_of_message() );;

        errno = (condor_errno_t)0;
        ret_val = (void * ) CONDOR_NotSupported( CONDOR_mmap );
        terrno = (condor_errno_t)errno;
        dprintf( D_SYSCALLS, "\tret_val = %p, errno = %d\n", ret_val, (int)terrno);

        syscall_sock->encode();
        assert( syscall_sock->code(ret_val) );
        // MAP_FAILED != NULL
        if( ret_val == MAP_FAILED ) {
            assert( syscall_sock->code(terrno) );
        }
        assert( syscall_sock->end_of_message() );;
        return 0;
	  }
#endif // defined( LINUX )
	case CONDOR_open:
	  {
		int   ret_val;
		char *  path;
		open_flags_t   flags;
		long   lastarg;
		condor_errno_t terrno;

		assert( syscall_sock->code(flags) );
		dprintf( D_SYSCALLS, "	flags = %d\n", flags );
		assert( syscall_sock->code(lastarg) );
		path = NULL;
		assert( syscall_sock->code(path) );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )open( path , flags , lastarg);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		free( (char *)path );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_close:
	  {
		int   ret_val;
		int   fd;
		condor_errno_t terrno;

		assert( syscall_sock->code(fd) );
		dprintf( D_SYSCALLS, "	fd = %d\n", fd );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )close( fd);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_unlink:
	  {
		int   ret_val;
		char *  path;
		condor_errno_t terrno;

		path = NULL;
		assert( syscall_sock->code(path) );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )unlink( path);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		free( (char *)path );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_rename:
	  {
		int   ret_val;
		char *  from;
		char *  to;
		condor_errno_t terrno;

		to = NULL;
		from = NULL;
		assert( syscall_sock->code(to) );
		assert( syscall_sock->code(from) );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )pseudo_rename( from , to);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		free( (char *)to );
		free( (char *)from );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_read:
	  {
		ssize_t   ret_val;
		int   fd;
		void *  buf;
		size_t   len;
		condor_errno_t terrno;

		assert( syscall_sock->code(fd) );
		dprintf( D_SYSCALLS, "	fd = %d\n", fd );
		assert( syscall_sock->code(len) );
		dprintf( D_SYSCALLS, "	len = %lu\n", len );
		buf = (void *)malloc( (unsigned)len );
		memset( buf, 0, (unsigned)len );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (ssize_t  )read( fd , buf , len);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %ld, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		if( !( ret_val < 0 ) ) {
			assert( syscall_sock->code_bytes_bool(buf, ret_val) );
		}
		free( (char *)buf );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_write:
	  {
		ssize_t   ret_val;
		int   fd;
		void *  buf;
		size_t   len;
		condor_errno_t terrno;

		assert( syscall_sock->code(fd) );
		dprintf( D_SYSCALLS, "	fd = %d\n", fd );
		assert( syscall_sock->code(len) );
		dprintf( D_SYSCALLS, "	len = %lu\n", len );
		buf = (void *)malloc( (unsigned)len );
		memset( buf, 0, (unsigned)len );
		assert( syscall_sock->code_bytes_bool(buf, len) );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (ssize_t  )write( fd , buf , len);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %ld, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		free( (char *)buf );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_lseek:
	case CONDOR_lseek64:
	case CONDOR_llseek:
	  {
		off_t   ret_val;
		int   fd;
		off_t   offset;
		int   whence;
		condor_errno_t terrno;

		assert( syscall_sock->code(fd) );
		dprintf( D_SYSCALLS, "	fd = %d\n", fd );
		assert( syscall_sock->code(offset) );
		dprintf( D_SYSCALLS, "	offset = %lu\n", offset );
		assert( syscall_sock->code(whence) );
		dprintf( D_SYSCALLS, "	whence = %d\n", whence );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (off_t  )lseek( fd , offset , whence);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %ld, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_dup:
	  {
		int   ret_val;
		int   old_fd;
		condor_errno_t terrno;

		assert( syscall_sock->code(old_fd) );
		dprintf( D_SYSCALLS, "	old_fd = %d\n", old_fd );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )dup( old_fd);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_dup2:
	  {
		int   ret_val;
		int   old_fd;
		int   new_fd;
		condor_errno_t terrno;

		assert( syscall_sock->code(old_fd) );
		dprintf( D_SYSCALLS, "	old_fd = %d\n", old_fd );
		assert( syscall_sock->code(new_fd) );
		dprintf( D_SYSCALLS, "	new_fd = %d\n", new_fd );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )dup2( old_fd , new_fd);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_fcntl:
	  {
		int   ret_val;
		int   fd;
		fcntl_cmd_t   cmd;
		long   lastarg;
		condor_errno_t terrno;

		assert( syscall_sock->code(fd) );
		dprintf( D_SYSCALLS, "	fd = %d\n", fd );
		assert( syscall_sock->code(cmd) );
		assert( syscall_sock->code(lastarg) );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )fcntl( fd , cmd , lastarg);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_ioctl:
	  {
		int   ret_val;
		int   fd;
		unsigned long   cmd;
		long   lastarg;
		condor_errno_t terrno;

		assert( syscall_sock->code(fd) );
		dprintf( D_SYSCALLS, "	fd = %d\n", fd );
		assert( syscall_sock->code(cmd) );
		assert( syscall_sock->code(lastarg) );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )ioctl( fd , cmd , lastarg);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_pipe:
	  {
		int   ret_val;
		int  *  fd;
		condor_errno_t terrno;

		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )pipe( fd);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_fsync:
	  {
		int   ret_val;
		int   fd;
		condor_errno_t terrno;

		assert( syscall_sock->code(fd) );
		dprintf( D_SYSCALLS, "	fd = %d\n", fd );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )fsync( fd);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_sync:
	  {
		int   ret_val;
		condor_errno_t terrno;

		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )pseudo_sync( );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_ftruncate:
	case CONDOR_ftruncate64:
	  {
		int   ret_val;
		int   fd;
		off_t   length;
		condor_errno_t terrno;

		assert( syscall_sock->code(fd) );
		dprintf( D_SYSCALLS, "	fd = %d\n", fd );
		assert( syscall_sock->code(length) );
		dprintf( D_SYSCALLS, "	length = %lu\n", length );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )ftruncate( fd , length);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_fchdir:
	  {
		int   ret_val;
		int   fd;
		condor_errno_t terrno;

		assert( syscall_sock->code(fd) );
		dprintf( D_SYSCALLS, "	fd = %d\n", fd );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )fchdir( fd);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_chdir:
	  {
		int   ret_val;
		char *  path;
		condor_errno_t terrno;

		path = NULL;
		assert( syscall_sock->code(path) );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )pseudo_chdir( path);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		free( (char *)path );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_socket:
	  {
		int   ret_val;
		int   domain;
		int   type;
		int   protocol;
		condor_errno_t terrno;

		assert( syscall_sock->code(domain) );
		dprintf( D_SYSCALLS, "	domain = %d\n", domain );
		assert( syscall_sock->code(type) );
		dprintf( D_SYSCALLS, "	type = %d\n", type );
		assert( syscall_sock->code(protocol) );
		dprintf( D_SYSCALLS, "	protocol = %d\n", protocol );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )socket( domain , type , protocol);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_fstat:
	case CONDOR_fstat64:
	  {
		int   ret_val;
		int   fd;
		struct stat *  buf;
		condor_errno_t terrno;

		assert( syscall_sock->code(fd) );
		dprintf( D_SYSCALLS, "	fd = %d\n", fd );
		buf = (struct stat *)malloc( (unsigned)STAT_SIZE );
		memset( buf, 0, (unsigned)STAT_SIZE );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )fstat( fd , buf);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		if( !( ret_val < 0 ) ) {
			assert( syscall_sock->code(buf) );
		}
		free( (char *)buf );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_stat:
	case CONDOR_stat64:
	  {
		int   ret_val;
		char *  name;
		struct stat *  buf;
		condor_errno_t terrno;

		name = NULL;
		buf = (struct stat *)malloc( (unsigned)STAT_SIZE );
		memset( buf, 0, (unsigned)STAT_SIZE );
		assert( syscall_sock->code(name) );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )stat( name , buf);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		if( !( ret_val < 0 ) ) {
			assert( syscall_sock->code(buf) );
		}
		free( (char *)buf );
		free( (char *)name );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_lstat:
	case CONDOR_lstat64:
	  {
		int   ret_val;
		char *  name;
		struct stat *  buf;
		condor_errno_t terrno;

		name = NULL;
		buf = (struct stat *)malloc( (unsigned)STAT_SIZE );
		memset( buf, 0, (unsigned)STAT_SIZE );
		assert( syscall_sock->code(name) );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )lstat( name , buf);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		if( !( ret_val < 0 ) ) {
			assert( syscall_sock->code(buf) );
		}
		free( (char *)buf );
		free( (char *)name );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_fchown:
	  {
		int   ret_val;
		int   fd;
		uid_t   owner;
		gid_t   group;
		condor_errno_t terrno;

		assert( syscall_sock->code(fd) );
		dprintf( D_SYSCALLS, "	fd = %d\n", fd );
		assert( syscall_sock->code(owner) );
		assert( syscall_sock->code(group) );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )fchown( fd , owner , group);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_fchmod:
	  {
		int   ret_val;
		int   fd;
		mode_t   mode;
		condor_errno_t terrno;

		assert( syscall_sock->code(fd) );
		dprintf( D_SYSCALLS, "	fd = %d\n", fd );
		assert( syscall_sock->code(mode) );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )fchmod( fd , mode);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_statfs:
	  {
		int   ret_val;
		char *  path;
		struct statfs *  buf;
		condor_errno_t terrno;

		path = NULL;
		buf = (struct statfs *)malloc( (unsigned)STATFS_SIZE );
		memset( buf, 0, (unsigned)STATFS_SIZE );
		assert( syscall_sock->code(path) );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )pseudo_statfs( path , buf);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		if( !( ret_val < 0 ) ) {
			assert( syscall_sock->code(buf) );
		}
		free( (char *)buf );
		free( (char *)path );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_fstatfs:
	  {
		int   ret_val;
		int   fd;
		struct statfs *  buf;
		condor_errno_t terrno;

		assert( syscall_sock->code(fd) );
		dprintf( D_SYSCALLS, "	fd = %d\n", fd );
		buf = (struct statfs *)malloc( (unsigned)STATFS_SIZE );
		memset( buf, 0, (unsigned)STATFS_SIZE );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )fstatfs( fd , buf);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		if( !( ret_val < 0 ) ) {
			assert( syscall_sock->code(buf) );
		}
		free( (char *)buf );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_flock:
	  {
		int   ret_val;
		int   fd;
		int   op;
		condor_errno_t terrno;

		assert( syscall_sock->code(fd) );
		dprintf( D_SYSCALLS, "	fd = %d\n", fd );
		assert( syscall_sock->code(op) );
		dprintf( D_SYSCALLS, "	op = %d\n", op );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )flock( fd , op);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_lockf:
	case CONDOR_lockf64:
	  {
		int   ret_val;
		int   fd;
		int   op;
		off_t   where;
		condor_errno_t terrno;

		assert( syscall_sock->code(fd) );
		dprintf( D_SYSCALLS, "	fd = %d\n", fd );
		assert( syscall_sock->code(op) );
		dprintf( D_SYSCALLS, "	op = %d\n", op );
		assert( syscall_sock->code(where) );
		dprintf( D_SYSCALLS, "	where = %lu\n", where );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )lockf( fd , op , where);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_getdirentries:
	  {
		ssize_t   ret_val;
		int   fd;
		char *  buf;
		size_t   nbytes;
		off_t *  basep;
		condor_errno_t terrno;

		assert( syscall_sock->code(fd) );
		dprintf( D_SYSCALLS, "	fd = %d\n", fd );
		assert( syscall_sock->code(nbytes) );
		dprintf( D_SYSCALLS, "	nbytes = %lu\n", nbytes );
		basep = (off_t *)malloc( (unsigned)LONG_SIZE );
		memset( basep, 0, (unsigned)LONG_SIZE );
		buf = (char *)malloc( (unsigned)nbytes );
		memset( buf, 0, (unsigned)nbytes );
		assert( syscall_sock->code_bytes_bool(basep, EIGHT) );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (ssize_t  )getdirentries( fd , buf , nbytes , basep);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %ld, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		if( !( ret_val < 0 ) ) {
			assert( syscall_sock->code_bytes_bool(basep, EIGHT) );
			assert( syscall_sock->code_bytes_bool(buf, ret_val) );
		}
		free( (char *)basep );
		free( (char *)buf );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_getdents:
	  {
		int   ret_val;
		int   fd;
		struct dirent *  buf;
		size_t   nbytes;
		condor_errno_t terrno;

		assert( syscall_sock->code(fd) );
		dprintf( D_SYSCALLS, "	fd = %d\n", fd );
		assert( syscall_sock->code(nbytes) );
		dprintf( D_SYSCALLS, "	nbytes = %lu\n", nbytes );
		buf = (struct dirent *)malloc( (unsigned)nbytes );
		memset( buf, 0, (unsigned)nbytes );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )getdents( fd , buf , nbytes);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		if( !( ret_val < 0 ) ) {
			assert( syscall_sock->code_bytes_bool(buf, nbytes) );
		}
		free( (char *)buf );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_getdents64:
	  {
		int   ret_val;
		int   fd;
		struct dirent64 *  buf;
		size_t   nbytes;
		condor_errno_t terrno;

		assert( syscall_sock->code(fd) );
		dprintf( D_SYSCALLS, "	fd = %d\n", fd );
		assert( syscall_sock->code(nbytes) );
		dprintf( D_SYSCALLS, "	nbytes = %lu\n", nbytes );
		buf = (struct dirent64 *)malloc( (unsigned)nbytes );
		memset( buf, 0, (unsigned)nbytes );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )getdents64( fd , buf , nbytes);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		if( !( ret_val < 0 ) ) {
			assert( syscall_sock->code_bytes_bool(buf, nbytes) );
		}
		free( (char *)buf );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_lseekread:
	  {
		off_t   ret_val;
		int   fd;
		off_t   offset;
		int   whence;
		void *  buf;
		size_t   len;
		condor_errno_t terrno;

		assert( syscall_sock->code(fd) );
		dprintf( D_SYSCALLS, "	fd = %d\n", fd );
		assert( syscall_sock->code(offset) );
		dprintf( D_SYSCALLS, "	offset = %lu\n", offset );
		assert( syscall_sock->code(whence) );
		dprintf( D_SYSCALLS, "	whence = %d\n", whence );
		assert( syscall_sock->code(len) );
		dprintf( D_SYSCALLS, "	len = %lu\n", len );
		buf = (void *)malloc( (unsigned)len );
		memset( buf, 0, (unsigned)len );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (off_t  )pseudo_lseekread( fd , offset , whence , buf , len);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %ld, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		if( !( ret_val < 0 ) ) {
			assert( syscall_sock->code_bytes_bool(buf, len) );
		}
		free( (char *)buf );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_lseekwrite:
	  {
		off_t   ret_val;
		int   fd;
		off_t   offset;
		int   whence;
		void *  buf;
		size_t   len;
		condor_errno_t terrno;

		assert( syscall_sock->code(fd) );
		dprintf( D_SYSCALLS, "	fd = %d\n", fd );
		assert( syscall_sock->code(offset) );
		dprintf( D_SYSCALLS, "	offset = %lu\n", offset );
		assert( syscall_sock->code(whence) );
		dprintf( D_SYSCALLS, "	whence = %d\n", whence );
		assert( syscall_sock->code(len) );
		dprintf( D_SYSCALLS, "	len = %lu\n", len );
		buf = (void *)malloc( (unsigned)len );
		memset( buf, 0, (unsigned)len );
		assert( syscall_sock->code_bytes_bool(buf, len) );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (off_t  )pseudo_lseekwrite( fd , offset , whence , buf , len);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %ld, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		free( (char *)buf );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_socketpair:
	  {
		int   ret_val;
		int   domain;
		int   type;
		int   protocol;
		int *  fds;
		condor_errno_t terrno;

		assert( syscall_sock->code(domain) );
		dprintf( D_SYSCALLS, "	domain = %d\n", domain );
		assert( syscall_sock->code(type) );
		dprintf( D_SYSCALLS, "	type = %d\n", type );
		assert( syscall_sock->code(protocol) );
		dprintf( D_SYSCALLS, "	protocol = %d\n", protocol );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  ) CONDOR_NotSupported( CONDOR_socketpair );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_accept:
	  {
		int   ret_val;
		int   fd;
		struct sockaddr *  addr;
		socklen_t *  length;
		condor_errno_t terrno;

		assert( syscall_sock->code(fd) );
		dprintf( D_SYSCALLS, "	fd = %d\n", fd );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )accept( fd , addr , length);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_bind:
	  {
		int   ret_val;
		int   fd;
		struct sockaddr *  addr;
		socklen_t   length;
		condor_errno_t terrno;

		assert( syscall_sock->code(fd) );
		dprintf( D_SYSCALLS, "	fd = %d\n", fd );
		assert( syscall_sock->code(length) );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )bind( fd , addr , length);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_connect:
	  {
		int   ret_val;
		int   fd;
		struct sockaddr *  addr;
		socklen_t   length;
		condor_errno_t terrno;

		assert( syscall_sock->code(fd) );
		dprintf( D_SYSCALLS, "	fd = %d\n", fd );
		assert( syscall_sock->code(length) );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )connect( fd , addr , length);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_getpeername:
	  {
		int   ret_val;
		int   fd;
		struct sockaddr *  addr;
		socklen_t *  length;
		condor_errno_t terrno;

		assert( syscall_sock->code(fd) );
		dprintf( D_SYSCALLS, "	fd = %d\n", fd );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )getpeername( fd , addr , length);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_getsockname:
	  {
		int   ret_val;
		int   fd;
		struct sockaddr *  addr;
		socklen_t *  length;
		condor_errno_t terrno;

		assert( syscall_sock->code(fd) );
		dprintf( D_SYSCALLS, "	fd = %d\n", fd );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )getsockname( fd , addr , length);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_getsockopt:
	  {
		int   ret_val;
		int   fd;
		int   level;
		int   option;
		void *  value;
		socklen_t *  length;
		condor_errno_t terrno;

		assert( syscall_sock->code(fd) );
		dprintf( D_SYSCALLS, "	fd = %d\n", fd );
		assert( syscall_sock->code(level) );
		dprintf( D_SYSCALLS, "	level = %d\n", level );
		assert( syscall_sock->code(option) );
		dprintf( D_SYSCALLS, "	option = %d\n", option );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )getsockopt( fd , level , option , value , length);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_listen:
	  {
		int   ret_val;
		int   fd;
		int   count;
		condor_errno_t terrno;

		assert( syscall_sock->code(fd) );
		dprintf( D_SYSCALLS, "	fd = %d\n", fd );
		assert( syscall_sock->code(count) );
		dprintf( D_SYSCALLS, "	count = %d\n", count );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )listen( fd , count);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_recvfrom:
	  {
		ssize_t   ret_val;
		int   fd;
		void *  data;
		unsigned long   length;
		int   flags;
		struct sockaddr *  addr;
		socklen_t *  addrlength;
		condor_errno_t terrno;

		assert( syscall_sock->code(fd) );
		dprintf( D_SYSCALLS, "	fd = %d\n", fd );
		assert( syscall_sock->code(length) );
		assert( syscall_sock->code(flags) );
		dprintf( D_SYSCALLS, "	flags = %d\n", flags );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (ssize_t  )recvfrom( fd , data , length , flags , addr , addrlength);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %ld, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_recvmsg:
	  {
		ssize_t   ret_val;
		int   fd;
		struct msghdr *  msg;
		int   flags;
		condor_errno_t terrno;

		assert( syscall_sock->code(fd) );
		dprintf( D_SYSCALLS, "	fd = %d\n", fd );
		assert( syscall_sock->code(flags) );
		dprintf( D_SYSCALLS, "	flags = %d\n", flags );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (ssize_t  )recvmsg( fd , msg , flags);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %ld, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_sendmsg:
	  {
		ssize_t   ret_val;
		int   fd;
		struct msghdr *  msg;
		int   flags;
		condor_errno_t terrno;

		assert( syscall_sock->code(fd) );
		dprintf( D_SYSCALLS, "	fd = %d\n", fd );
		assert( syscall_sock->code(flags) );
		dprintf( D_SYSCALLS, "	flags = %d\n", flags );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (ssize_t  )sendmsg( fd , msg , flags);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %ld, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_sendto:
	  {
		ssize_t   ret_val;
		int   fd;
		void *  data;
		unsigned long   length;
		int   flags;
		struct sockaddr *  addr;
		socklen_t   addrlength;
		condor_errno_t terrno;

		assert( syscall_sock->code(fd) );
		dprintf( D_SYSCALLS, "	fd = %d\n", fd );
		assert( syscall_sock->code(length) );
		assert( syscall_sock->code(flags) );
		dprintf( D_SYSCALLS, "	flags = %d\n", flags );
		assert( syscall_sock->code(addrlength) );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (ssize_t  )sendto( fd , data , length , flags , addr , addrlength);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %ld, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_setsockopt:
	  {
		int   ret_val;
		int   fd;
		int   level;
		int   option;
		void *  value;
		socklen_t   length;
		condor_errno_t terrno;

		assert( syscall_sock->code(fd) );
		dprintf( D_SYSCALLS, "	fd = %d\n", fd );
		assert( syscall_sock->code(level) );
		dprintf( D_SYSCALLS, "	level = %d\n", level );
		assert( syscall_sock->code(option) );
		dprintf( D_SYSCALLS, "	option = %d\n", option );
		assert( syscall_sock->code(length) );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )setsockopt( fd , level , option , value , length);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_shutdown:
	  {
		int   ret_val;
		int   fd;
		int   how;
		condor_errno_t terrno;

		assert( syscall_sock->code(fd) );
		dprintf( D_SYSCALLS, "	fd = %d\n", fd );
		assert( syscall_sock->code(how) );
		dprintf( D_SYSCALLS, "	how = %d\n", how );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )shutdown( fd , how);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_link:
	  {
		int   ret_val;
		char *  p1;
		char *  p2;
		condor_errno_t terrno;

		p2 = NULL;
		p1 = NULL;
		assert( syscall_sock->code(p2) );
		assert( syscall_sock->code(p1) );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )link( p1 , p2);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		free( (char *)p2 );
		free( (char *)p1 );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_access:
	  {
		int   ret_val;
		char *  path;
		int   access_mode;
		condor_errno_t terrno;

		assert( syscall_sock->code(access_mode) );
		dprintf( D_SYSCALLS, "	access_mode = %d\n", access_mode );
		path = NULL;
		assert( syscall_sock->code(path) );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )access( path , access_mode);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		free( (char *)path );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_mknod:
	  {
		int   ret_val;
		char *  path;
		mode_t   mode;
		dev_t   device;
		condor_errno_t terrno;

		assert( syscall_sock->code(mode) );
		assert( syscall_sock->code(device) );
		path = NULL;
		assert( syscall_sock->code(path) );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )mknod( path , mode , device);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		free( (char *)path );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_chmod:
	  {
		int   ret_val;
		char *  path;
		mode_t   mode;
		condor_errno_t terrno;

		assert( syscall_sock->code(mode) );
		path = NULL;
		assert( syscall_sock->code(path) );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )chmod( path , mode);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		free( (char *)path );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_symlink:
	  {
		int   ret_val;
		char *  path1;
		char *  path2;
		condor_errno_t terrno;

		path2 = NULL;
		path1 = NULL;
		assert( syscall_sock->code(path2) );
		assert( syscall_sock->code(path1) );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )symlink( path1 , path2);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		free( (char *)path2 );
		free( (char *)path1 );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_readlink:
	  {
		ssize_t   ret_val;
		char *  path;
		char *  buf;
		size_t   buf_size;
		condor_errno_t terrno;

		assert( syscall_sock->code(buf_size) );
		dprintf( D_SYSCALLS, "	buf_size = %lu\n", buf_size );
		path = NULL;
		buf = (char *)malloc( (unsigned)buf_size );
		memset( buf, 0, (unsigned)buf_size );
		assert( syscall_sock->code(path) );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (ssize_t  )readlink( path , buf , buf_size);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %ld, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		if( !( ret_val < 0 ) ) {
			assert( syscall_sock->code_bytes_bool(buf, buf_size) );
		}
		free( (char *)buf );
		free( (char *)path );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_chown:
	  {
		int   ret_val;
		char *  path;
		uid_t   owner;
		gid_t   group;
		condor_errno_t terrno;

		assert( syscall_sock->code(owner) );
		assert( syscall_sock->code(group) );
		path = NULL;
		assert( syscall_sock->code(path) );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )chown( path , owner , group);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		free( (char *)path );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_truncate:
	  {
		int   ret_val;
		char *  path;
		off_t   length;
		condor_errno_t terrno;

		assert( syscall_sock->code(length) );
		dprintf( D_SYSCALLS, "	length = %lu\n", length );
		path = NULL;
		assert( syscall_sock->code(path) );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )truncate( path , length);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		free( (char *)path );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_mkdir:
	  {
		int   ret_val;
		char *  path;
		mode_t   mode;
		condor_errno_t terrno;

		assert( syscall_sock->code(mode) );
		path = NULL;
		assert( syscall_sock->code(path) );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )mkdir( path , mode);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		free( (char *)path );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_rmdir:
	  {
		int   ret_val;
		char *  path;
		condor_errno_t terrno;

		path = NULL;
		assert( syscall_sock->code(path) );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )rmdir( path);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		free( (char *)path );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_utimes:
	  {
		int   ret_val;
		char *  path;
		struct timeval *  times;
		condor_errno_t terrno;

		path = NULL;
		times = (struct timeval *)malloc( (unsigned)TIMEVAL_SIZE );
		memset( times, 0, (unsigned)TIMEVAL_SIZE );
		assert( syscall_sock->code(times) );
		assert( syscall_sock->code(path) );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )utimes( path , times);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		free( (char *)times );
		free( (char *)path );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_utime:
	  {
		int   ret_val;
		char *  path;
		struct utimbuf *  buf;
		condor_errno_t terrno;

		path = NULL;
		buf = (struct utimbuf *)malloc( (unsigned)UTIMBUF_SIZE );
		memset( buf, 0, (unsigned)UTIMBUF_SIZE );
		assert( syscall_sock->code(buf) );
		assert( syscall_sock->code(path) );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )utime( path , buf);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		free( (char *)buf );
		free( (char *)path );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_shell:
	  {
		int   ret_val;
		char *  command;
		int   len;
		condor_errno_t terrno;

		assert( syscall_sock->code(len) );
		dprintf( D_SYSCALLS, "	len = %d\n", len );
		command = (char *)malloc( (unsigned)len );
		memset( command, 0, (unsigned)len );
		assert( syscall_sock->code_bytes_bool(command, len) );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )pseudo_shell( command , len);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		free( (char *)command );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_getwd_special:
	case CONDOR_getwd:
	  {
		int   ret_val;
		char *  path_name;
		condor_errno_t terrno;

		path_name = NULL;
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )pseudo_getwd_special( path_name);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		if( !( ret_val < 0 ) ) {
			assert( syscall_sock->code(path_name) );
		}
		free( (char *)path_name );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_getuid:
	  {
		uid_t   ret_val;
		condor_errno_t terrno;

		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (uid_t  )pseudo_getuid( );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_getgid:
	  {
		gid_t   ret_val;
		condor_errno_t terrno;

		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (gid_t  )pseudo_getgid( );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_getppid:
	  {
		pid_t   ret_val;
		condor_errno_t terrno;

		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (pid_t  )pseudo_getppid( );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_getpid:
	  {
		pid_t   ret_val;
		condor_errno_t terrno;

		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (pid_t  )pseudo_getpid( );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_getpgid:
	  {
		pid_t   ret_val;
		pid_t   pid;
		condor_errno_t terrno;

		assert( syscall_sock->code(pid) );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (pid_t  )getpgid( pid);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_getpgrp:
	  {
		pid_t   ret_val;
		condor_errno_t terrno;

		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (pid_t  )getpgrp( );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_getlogin:
	  {
		int   ret_val;
		char *  loginbuf;
		condor_errno_t terrno;

		loginbuf = NULL;
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )pseudo_getlogin( loginbuf);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		if( !( ret_val < 0 ) ) {
			assert( syscall_sock->code(loginbuf) );
		}
		free( (char *)loginbuf );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_umask:
	  {
		mode_t   ret_val;
		mode_t   cmask;
		condor_errno_t terrno;

		assert( syscall_sock->code(cmask) );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (mode_t  )umask( cmask);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_getpagesize:
	  {
		int   ret_val;
		condor_errno_t terrno;

		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )getpagesize( );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_getgroups:
	  {
		int   ret_val;
		int   gidsetsize;
		gid_t  *  grouplist;
		condor_errno_t terrno;

		assert( syscall_sock->code(gidsetsize) );
		dprintf( D_SYSCALLS, "	gidsetsize = %d\n", gidsetsize );
		grouplist = (gid_t *)malloc( (unsigned)gidsetsize * GID_T_SIZE );
		memset( grouplist, 0, (unsigned)gidsetsize * GID_T_SIZE );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )getgroups( gidsetsize , grouplist);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		if( !( ret_val < 0 ) ) {
			assert( syscall_sock->code_array(grouplist, gidsetsize) );
		}
		free( (char *)grouplist );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_gethostname:
	  {
		int   ret_val;
		char *  address;
		size_t   address_len;
		condor_errno_t terrno;

		assert( syscall_sock->code(address_len) );
		dprintf( D_SYSCALLS, "	address_len = %lu\n", address_len );
		address = (char *)malloc( (unsigned)address_len );
		memset( address, 0, (unsigned)address_len );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )gethostname( address , address_len);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		if( !( ret_val < 0 ) ) {
			assert( syscall_sock->code_bytes_bool(address, address_len) );
		}
		free( (char *)address );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_getdtablesize:
	  {
		int   ret_val;
		condor_errno_t terrno;

		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )getdtablesize( );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_setpriority:
	  {
		int   ret_val;
		int   which;
		int   who;
		int   priority;
		condor_errno_t terrno;

		assert( syscall_sock->code(which) );
		dprintf( D_SYSCALLS, "	which = %d\n", which );
		assert( syscall_sock->code(who) );
		dprintf( D_SYSCALLS, "	who = %d\n", who );
		assert( syscall_sock->code(priority) );
		dprintf( D_SYSCALLS, "	priority = %d\n", priority );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )setpriority( which , who , priority);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_getpriority:
	  {
		int   ret_val;
		int   which;
		int   who;
		condor_errno_t terrno;

		assert( syscall_sock->code(which) );
		dprintf( D_SYSCALLS, "	which = %d\n", which );
		assert( syscall_sock->code(who) );
		dprintf( D_SYSCALLS, "	who = %d\n", who );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )getpriority( which , who);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_gettimeofday:
	  {
		int   ret_val;
		struct timeval *  tp;
		struct timezone *  tzp;
		condor_errno_t terrno;

		tzp = (struct timezone *)malloc( (unsigned)TIMEZONE_SIZE );
		memset( tzp, 0, (unsigned)TIMEZONE_SIZE );
		tp = (struct timeval *)malloc( (unsigned)TIMEVAL_SIZE );
		memset( tp, 0, (unsigned)TIMEVAL_SIZE );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )gettimeofday( tp , tzp);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		if( !( ret_val < 0 ) ) {
			assert( syscall_sock->code(tp) );
		}
		free( (char *)tzp );
		free( (char *)tp );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_gethostid:
	  {
		int   ret_val;
		condor_errno_t terrno;

		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )gethostid( );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_setrlimit:
	  {
		int   ret_val;
		int   resource;
		struct rlimit *  rlp;
		condor_errno_t terrno;

		assert( syscall_sock->code(resource) );
		dprintf( D_SYSCALLS, "	resource = %d\n", resource );
		rlp = (struct rlimit *)malloc( (unsigned)RLIMIT_SIZE );
		memset( rlp, 0, (unsigned)RLIMIT_SIZE );
		assert( syscall_sock->code(rlp) );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )setrlimit( resource , rlp);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		free( (char *)rlp );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_getrlimit:
	  {
		int   ret_val;
		int   resource;
		struct rlimit *  rlp;
		condor_errno_t terrno;

		assert( syscall_sock->code(resource) );
		dprintf( D_SYSCALLS, "	resource = %d\n", resource );
		rlp = (struct rlimit *)malloc( (unsigned)RLIMIT_SIZE );
		memset( rlp, 0, (unsigned)RLIMIT_SIZE );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )getrlimit( resource , rlp);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		if( !( ret_val < 0 ) ) {
			assert( syscall_sock->code(rlp) );
		}
		free( (char *)rlp );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_getdomainname:
	  {
		int   ret_val;
		char *  name;
		int   namelen;
		condor_errno_t terrno;

		assert( syscall_sock->code(namelen) );
		dprintf( D_SYSCALLS, "	namelen = %d\n", namelen );
		name = (char *)malloc( (unsigned)namelen );
		memset( name, 0, (unsigned)namelen );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )getdomainname( name , namelen);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		if( !( ret_val < 0 ) ) {
			assert( syscall_sock->code_bytes_bool(name, namelen) );
		}
		free( (char *)name );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_mvalid:
	  {
		int   ret_val;
		char *  addr;
		size_t   len;
		int   prot;
		condor_errno_t terrno;

		assert( syscall_sock->code(len) );
		dprintf( D_SYSCALLS, "	len = %lu\n", len );
		assert( syscall_sock->code(prot) );
		dprintf( D_SYSCALLS, "	prot = %d\n", prot );
		addr = (char *)malloc( (unsigned)len );
		memset( addr, 0, (unsigned)len );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  ) CONDOR_NotSupported( CONDOR_mvalid );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		if( !( ret_val < 0 ) ) {
			assert( syscall_sock->code_bytes_bool(addr, len) );
		}
		free( (char *)addr );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_uswitch:
	  {
		int   ret_val;
		int   cmd;
		int   value;
		condor_errno_t terrno;

		assert( syscall_sock->code(cmd) );
		dprintf( D_SYSCALLS, "	cmd = %d\n", cmd );
		assert( syscall_sock->code(value) );
		dprintf( D_SYSCALLS, "	value = %d\n", value );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  ) CONDOR_NotSupported( CONDOR_uswitch );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_geteuid:
	  {
		uid_t   ret_val;
		condor_errno_t terrno;

		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (uid_t  )pseudo_geteuid( );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_getegid:
	  {
		gid_t   ret_val;
		condor_errno_t terrno;

		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (gid_t  )pseudo_getegid( );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_register_syscall_version:
	  {
		int   ret_val;
		char *  version;
		condor_errno_t terrno;

		version = NULL;
		assert( syscall_sock->code(version) );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )pseudo_register_syscall_version( version);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		free( (char *)version );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_register_ckpt_platform:
	  {
		int   ret_val;
		char *  platform;
		int   len;
		condor_errno_t terrno;

		assert( syscall_sock->code(len) );
		dprintf( D_SYSCALLS, "	len = %d\n", len );
		platform = (char *)malloc( (unsigned)len );
		memset( platform, 0, (unsigned)len );
		assert( syscall_sock->code_bytes_bool(platform, len) );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )pseudo_register_ckpt_platform( platform , len);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		free( (char *)platform );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_register_fs_domain:
	  {
		int   ret_val;
		char *  domain;
		condor_errno_t terrno;

		domain = NULL;
		assert( syscall_sock->code(domain) );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )pseudo_register_fs_domain( domain);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		free( (char *)domain );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_register_uid_domain:
	  {
		int   ret_val;
		char *  domain;
		condor_errno_t terrno;

		domain = NULL;
		assert( syscall_sock->code(domain) );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )pseudo_register_uid_domain( domain);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		free( (char *)domain );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_get_universe:
	  {
		int   ret_val;
		int *  univ_id;
		condor_errno_t terrno;

		univ_id = (int *)malloc( (unsigned)INT_SIZE );
		memset( univ_id, 0, (unsigned)INT_SIZE );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )pseudo_get_universe( univ_id);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		if( !( ret_val < 0 ) ) {
			assert( syscall_sock->code(univ_id) );
		}
		free( (char *)univ_id );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_get_username:
	  {
		int   ret_val;
		char *  uname;
		condor_errno_t terrno;

		uname = NULL;
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )pseudo_get_username( uname);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		if( !( ret_val < 0 ) ) {
			assert( syscall_sock->code(uname) );
		}
		free( (char *)uname );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_extern_name:
	  {
		int   ret_val;
		char *  path;
		char *  buf;
		int   bufsize;
		condor_errno_t terrno;

		assert( syscall_sock->code(bufsize) );
		dprintf( D_SYSCALLS, "	bufsize = %d\n", bufsize );
		path = NULL;
		buf = (char *)malloc( (unsigned)bufsize );
		memset( buf, 0, (unsigned)bufsize );
		assert( syscall_sock->code(path) );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )pseudo_extern_name( path , buf , bufsize);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		if( !( ret_val < 0 ) ) {
			assert( syscall_sock->code_bytes_bool(buf, bufsize) );
		}
		free( (char *)buf );
		free( (char *)path );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_free_fs_blocks:
	  {
		int   ret_val;
		char *  pathname;
		condor_errno_t terrno;

		pathname = NULL;
		assert( syscall_sock->code(pathname) );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )pseudo_free_fs_blocks( pathname);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		free( (char *)pathname );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_reallyexit:
	  {
		int   ret_val;
		int *  status;
		struct rusage *  use_p;
		condor_errno_t terrno;

		use_p = (struct rusage *)malloc( (unsigned)RUSAGE_SIZE );
		memset( use_p, 0, (unsigned)RUSAGE_SIZE );
		status = (int *)malloc( (unsigned)INT_SIZE );
		memset( status, 0, (unsigned)INT_SIZE );
		assert( syscall_sock->code(use_p) );
		assert( syscall_sock->code(status) );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )pseudo_reallyexit( status , use_p);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		free( (char *)use_p );
		free( (char *)status );
		assert( syscall_sock->end_of_message() );;
		return NEG_ONE;
	}

	case CONDOR_subproc_status:
	  {
		int   ret_val;
		int   id;
		int *  status;
		struct rusage *  use_p;
		condor_errno_t terrno;

		assert( syscall_sock->code(id) );
		dprintf( D_SYSCALLS, "	id = %d\n", id );
		use_p = (struct rusage *)malloc( (unsigned)RUSAGE_SIZE );
		memset( use_p, 0, (unsigned)RUSAGE_SIZE );
		status = (int *)malloc( (unsigned)INT_SIZE );
		memset( status, 0, (unsigned)INT_SIZE );
		assert( syscall_sock->code(use_p) );
		assert( syscall_sock->code(status) );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )pseudo_subproc_status( id , status , use_p);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		free( (char *)use_p );
		free( (char *)status );
		assert( syscall_sock->end_of_message() );;
		return NEG_ONE;
	}

	case CONDOR_image_size:
	  {
		int   ret_val;
		int   kbytes;
		condor_errno_t terrno;

		assert( syscall_sock->code(kbytes) );
		dprintf( D_SYSCALLS, "	kbytes = %d\n", kbytes );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )pseudo_image_size( kbytes);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_send_rusage:
	  {
		int   ret_val;
		struct rusage *  use_p;
		condor_errno_t terrno;

		use_p = (struct rusage *)malloc( (unsigned)RUSAGE_SIZE );
		memset( use_p, 0, (unsigned)RUSAGE_SIZE );
		assert( syscall_sock->code(use_p) );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )pseudo_send_rusage( use_p);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		free( (char *)use_p );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_getrusage:
	  {
		int   ret_val;
		int   who;
		struct rusage *  use_p;
		condor_errno_t terrno;

		assert( syscall_sock->code(who) );
		dprintf( D_SYSCALLS, "	who = %d\n", who );
		use_p = (struct rusage *)malloc( (unsigned)RUSAGE_SIZE );
		memset( use_p, 0, (unsigned)RUSAGE_SIZE );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )pseudo_getrusage( who , use_p);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		if( !( ret_val < 0 ) ) {
			assert( syscall_sock->code(use_p) );
		}
		free( (char *)use_p );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_report_error:
	  {
		int   ret_val;
		char *  msg;
		condor_errno_t terrno;

		msg = NULL;
		assert( syscall_sock->code(msg) );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )pseudo_report_error( msg);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		free( (char *)msg );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_report_file_info:
	  {
		int   ret_val;
		char *  kind;
		char *  name;
		int   open_count;
		int   read_count;
		int   write_count;
		int   seek_count;
		int   read_bytes;
		int   write_bytes;
		condor_errno_t terrno;

		assert( syscall_sock->code(open_count) );
		dprintf( D_SYSCALLS, "	open_count = %d\n", open_count );
		assert( syscall_sock->code(read_count) );
		dprintf( D_SYSCALLS, "	read_count = %d\n", read_count );
		assert( syscall_sock->code(write_count) );
		dprintf( D_SYSCALLS, "	write_count = %d\n", write_count );
		assert( syscall_sock->code(seek_count) );
		dprintf( D_SYSCALLS, "	seek_count = %d\n", seek_count );
		assert( syscall_sock->code(read_bytes) );
		dprintf( D_SYSCALLS, "	read_bytes = %d\n", read_bytes );
		assert( syscall_sock->code(write_bytes) );
		dprintf( D_SYSCALLS, "	write_bytes = %d\n", write_bytes );
		name = NULL;
		kind = NULL;
		assert( syscall_sock->code(name) );
		assert( syscall_sock->code(kind) );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )pseudo_report_file_info( kind , name , open_count , read_count , write_count , seek_count , read_bytes , write_bytes);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		free( (char *)name );
		free( (char *)kind );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_report_file_info_new:
	  {
		int   ret_val;
		char *  name;
		int64_t   open_count;
		int64_t   read_count;
		int64_t   write_count;
		int64_t   seek_count;
		int64_t   read_bytes;
		int64_t   write_bytes;
		condor_errno_t terrno;

		assert( syscall_sock->code(open_count) );
		assert( syscall_sock->code(read_count) );
		assert( syscall_sock->code(write_count) );
		assert( syscall_sock->code(seek_count) );
		assert( syscall_sock->code(read_bytes) );
		assert( syscall_sock->code(write_bytes) );
		name = NULL;
		assert( syscall_sock->code(name) );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )pseudo_report_file_info_new( name , open_count , read_count , write_count , seek_count , read_bytes , write_bytes);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		free( (char *)name );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_get_std_file_info:
	  {
		int   ret_val;
		int   fd;
		char *  logical_name;
		condor_errno_t terrno;

		assert( syscall_sock->code(fd) );
		dprintf( D_SYSCALLS, "	fd = %d\n", fd );
		logical_name = NULL;
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )pseudo_get_std_file_info( fd , logical_name);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		if( !( ret_val < 0 ) ) {
			assert( syscall_sock->code(logical_name) );
		}
		free( (char *)logical_name );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_get_file_info_new:
	  {
		int   ret_val;
		char *  logical_name;
		char *  actual_url;
		condor_errno_t terrno;

		actual_url = NULL;
		logical_name = NULL;
		assert( syscall_sock->code(logical_name) );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )pseudo_get_file_info_new( logical_name , actual_url);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		if( !( ret_val < 0 ) ) {
			assert( syscall_sock->code(actual_url) );
		}
		free( (char *)actual_url );
		free( (char *)logical_name );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_get_file_info:
	  {
		int   ret_val;
		char *  logical_name;
		char *  actual_url;
		condor_errno_t terrno;

		actual_url = NULL;
		logical_name = NULL;
		assert( syscall_sock->code(logical_name) );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )pseudo_get_file_info( logical_name , actual_url);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		if( !( ret_val < 0 ) ) {
			assert( syscall_sock->code(actual_url) );
		}
		free( (char *)actual_url );
		free( (char *)logical_name );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_std_file_info:
	  {
		int   ret_val;
		int   fd;
		char *  logical_name;
		int *  pipe_fd;
		condor_errno_t terrno;

		assert( syscall_sock->code(fd) );
		dprintf( D_SYSCALLS, "	fd = %d\n", fd );
		logical_name = NULL;
		pipe_fd = (int *)malloc( (unsigned)INT_SIZE );
		memset( pipe_fd, 0, (unsigned)INT_SIZE );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )pseudo_std_file_info( fd , logical_name , pipe_fd);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		if( !( ret_val < 0 ) ) {
			assert( syscall_sock->code(pipe_fd) );
			assert( syscall_sock->code(logical_name) );
		}
		free( (char *)pipe_fd );
		free( (char *)logical_name );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_file_info:
	  {
		int   ret_val;
		char *  logical_name;
		int *  fd;
		char *  physical_name;
		condor_errno_t terrno;

		physical_name = NULL;
		logical_name = NULL;
		fd = (int *)malloc( (unsigned)INT_SIZE );
		memset( fd, 0, (unsigned)INT_SIZE );
		assert( syscall_sock->code(logical_name) );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )pseudo_file_info( logical_name , fd , physical_name);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		if( !( ret_val < 0 ) ) {
			assert( syscall_sock->code(fd) );
			assert( syscall_sock->code(physical_name) );
		}
		free( (char *)fd );
		free( (char *)physical_name );
		free( (char *)logical_name );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_get_buffer_info:
	  {
		int   ret_val;
		int *  bytes;
		int *  block_size;
		int *  prefetch_size;
		condor_errno_t terrno;

		prefetch_size = (int *)malloc( (unsigned)INT_SIZE );
		memset( prefetch_size, 0, (unsigned)INT_SIZE );
		block_size = (int *)malloc( (unsigned)INT_SIZE );
		memset( block_size, 0, (unsigned)INT_SIZE );
		bytes = (int *)malloc( (unsigned)INT_SIZE );
		memset( bytes, 0, (unsigned)INT_SIZE );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )pseudo_get_buffer_info( bytes , block_size , prefetch_size);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		if( !( ret_val < 0 ) ) {
			assert( syscall_sock->code(prefetch_size) );
			assert( syscall_sock->code(block_size) );
			assert( syscall_sock->code(bytes) );
		}
		free( (char *)prefetch_size );
		free( (char *)block_size );
		free( (char *)bytes );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_get_job_attr:
	  {
		int   ret_val;
		char *  attrname;
		char *  expr;
		condor_errno_t terrno;

		expr = NULL;
		attrname = NULL;
		assert( syscall_sock->code(attrname) );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )pseudo_get_job_attr( attrname , expr);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		if( !( ret_val < 0 ) ) {
			assert( syscall_sock->code(expr) );
		}
		free( (char *)expr );
		free( (char *)attrname );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_set_job_attr:
	  {
		int   ret_val;
		char *  attrname;
		char *  expr;
		condor_errno_t terrno;

		expr = NULL;
		attrname = NULL;
		assert( syscall_sock->code(expr) );
		assert( syscall_sock->code(attrname) );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )pseudo_set_job_attr( attrname , expr);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		free( (char *)expr );
		free( (char *)attrname );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_constrain:
	  {
		int   ret_val;
		char *  expr;
		condor_errno_t terrno;

		expr = NULL;
		assert( syscall_sock->code(expr) );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )pseudo_constrain( expr);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		free( (char *)expr );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_get_iwd:
	  {
		int   ret_val;
		char *  path;
		condor_errno_t terrno;

		path = NULL;
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )pseudo_get_iwd( path);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		if( !( ret_val < 0 ) ) {
			assert( syscall_sock->code(path) );
		}
		free( (char *)path );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_get_ckpt_name:
	  {
		int   ret_val;
		char *  path;
		condor_errno_t terrno;

		path = NULL;
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )pseudo_get_ckpt_name( path);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		if( !( ret_val < 0 ) ) {
			assert( syscall_sock->code(path) );
		}
		free( (char *)path );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_get_ckpt_mode:
	  {
		int   ret_val;
		condor_signal_t   sig;
		condor_errno_t terrno;

		assert( syscall_sock->code(sig) );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )pseudo_get_ckpt_mode( sig);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_get_ckpt_speed:
	  {
		int   ret_val;
		condor_errno_t terrno;

		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )pseudo_get_ckpt_speed( );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_get_a_out_name:
	  {
		int   ret_val;
		char *  path;
		condor_errno_t terrno;

		path = NULL;
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )pseudo_get_a_out_name( path);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		if( !( ret_val < 0 ) ) {
			assert( syscall_sock->code(path) );
		}
		free( (char *)path );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_pvm_info:
	  {
		int   ret_val;
		char *  response;
		int   mytid;
		condor_errno_t terrno;

		assert( syscall_sock->code(mytid) );
		dprintf( D_SYSCALLS, "	mytid = %d\n", mytid );
		response = NULL;
		assert( syscall_sock->code(response) );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )pseudo_pvm_info( response , mytid);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		free( (char *)response );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_pvm_task_info:
	  {
		int   ret_val;
		int   v_pid;
		int   pvm_tid;
		condor_errno_t terrno;

		assert( syscall_sock->code(v_pid) );
		dprintf( D_SYSCALLS, "	v_pid = %d\n", v_pid );
		assert( syscall_sock->code(pvm_tid) );
		dprintf( D_SYSCALLS, "	pvm_tid = %d\n", pvm_tid );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )pseudo_pvm_task_info( v_pid , pvm_tid);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_suspended:
	  {
		int   ret_val;
		int   susp;
		condor_errno_t terrno;

		assert( syscall_sock->code(susp) );
		dprintf( D_SYSCALLS, "	susp = %d\n", susp );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )pseudo_suspended( susp);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_get_file_stream:
	  {
		int   ret_val;
		char *  file;
		size_t *  len;
		unsigned int *  ip_addr;
		u_short *  port_num;
		condor_errno_t terrno;

		file = NULL;
		port_num = (u_short *)malloc( (unsigned)U_SHORT_SIZE );
		memset( port_num, 0, (unsigned)U_SHORT_SIZE );
		ip_addr = (unsigned int *)malloc( (unsigned)U_INT_SIZE );
		memset( ip_addr, 0, (unsigned)U_INT_SIZE );
		len = (size_t *)malloc( (unsigned)SIZE_T_SIZE );
		memset( len, 0, (unsigned)SIZE_T_SIZE );
		assert( syscall_sock->code(file) );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )pseudo_get_file_stream( file , len , ip_addr , port_num);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		if( !( ret_val < 0 ) ) {
			assert( syscall_sock->code(port_num) );
			assert( syscall_sock->code(ip_addr) );
			assert( syscall_sock->code(len) );
		}
		free( (char *)port_num );
		free( (char *)ip_addr );
		free( (char *)len );
		free( (char *)file );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_put_file_stream:
	  {
		int   ret_val;
		char *  file;
		size_t   len;
		unsigned int *  ip_addr;
		u_short *  port_num;
		condor_errno_t terrno;

		assert( syscall_sock->code(len) );
		dprintf( D_SYSCALLS, "	len = %lu\n", len );
		file = NULL;
		port_num = (u_short *)malloc( (unsigned)U_SHORT_SIZE );
		memset( port_num, 0, (unsigned)U_SHORT_SIZE );
		ip_addr = (unsigned int *)malloc( (unsigned)U_INT_SIZE );
		memset( ip_addr, 0, (unsigned)U_INT_SIZE );
		assert( syscall_sock->code(file) );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )pseudo_put_file_stream( file , len , ip_addr , port_num);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		if( !( ret_val < 0 ) ) {
			assert( syscall_sock->code(port_num) );
			assert( syscall_sock->code(ip_addr) );
		}
		free( (char *)port_num );
		free( (char *)ip_addr );
		free( (char *)file );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_register_ckpt_server:
	  {
		int   ret_val;
		char *  host;
		condor_errno_t terrno;

		host = NULL;
		assert( syscall_sock->code(host) );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )pseudo_register_ckpt_server( host);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		free( (char *)host );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_register_arch:
	  {
		int   ret_val;
		char *  arch;
		condor_errno_t terrno;

		arch = NULL;
		assert( syscall_sock->code(arch) );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )pseudo_register_arch( arch);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		free( (char *)arch );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_register_opsys:
	  {
		int   ret_val;
		char *  opsys;
		condor_errno_t terrno;

		opsys = NULL;
		assert( syscall_sock->code(opsys) );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )pseudo_register_opsys( opsys);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		free( (char *)opsys );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_register_begin_execution:
	  {
		int   ret_val;
		condor_errno_t terrno;

		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )pseudo_register_begin_execution( );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_send_a_file:
	  {
		int   ret_val;
		char *  name;
		mode_t   mode;
		condor_errno_t terrno;

		assert( syscall_sock->code(mode) );
		name = NULL;
		assert( syscall_sock->code(name) );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )pseudo_send_a_file( name , mode);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		free( (char *)name );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_get_file:
	  {
		int   ret_val;
		char *  name;
		condor_errno_t terrno;

		name = NULL;
		assert( syscall_sock->code(name) );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  )pseudo_get_file( name);
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		free( (char *)name );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_mprotect:
	  {
		int   ret_val;
		MMAP_T   addr;
		size_t   len;
		int   prot;
		condor_errno_t terrno;

		assert( syscall_sock->code(addr) );
		assert( syscall_sock->code(len) );
		dprintf( D_SYSCALLS, "	len = %lu\n", len );
		assert( syscall_sock->code(prot) );
		dprintf( D_SYSCALLS, "	prot = %d\n", prot );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  ) CONDOR_NotSupported( CONDOR_mprotect );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_setuid:
	  {
		int   ret_val;
		uid_t   uid;
		condor_errno_t terrno;

		assert( syscall_sock->code(uid) );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  ) CONDOR_NotSupported( CONDOR_setuid );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_setgid:
	  {
		int   ret_val;
		gid_t   gid;
		condor_errno_t terrno;

		assert( syscall_sock->code(gid) );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  ) CONDOR_NotSupported( CONDOR_setgid );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_setreuid:
	  {
		int   ret_val;
		uid_t   ruid;
		uid_t   euid;
		condor_errno_t terrno;

		assert( syscall_sock->code(ruid) );
		assert( syscall_sock->code(euid) );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  ) CONDOR_NotSupported( CONDOR_setreuid );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_setregid:
	  {
		int   ret_val;
		uid_t   rgid;
		uid_t   egid;
		condor_errno_t terrno;

		assert( syscall_sock->code(rgid) );
		assert( syscall_sock->code(egid) );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  ) CONDOR_NotSupported( CONDOR_setregid );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_setlogin:
	  {
		int   ret_val;
		char *  name;
		condor_errno_t terrno;

		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  ) CONDOR_NotSupported( CONDOR_setlogin );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_revoke:
	  {
		int   ret_val;
		char *  path;
		condor_errno_t terrno;

		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  ) CONDOR_NotSupported( CONDOR_revoke );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_chroot:
	  {
		int   ret_val;
		char *  path;
		condor_errno_t terrno;

		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  ) CONDOR_NotSupported( CONDOR_chroot );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_setgroups:
	  {
		int   ret_val;
		int   gidsetsize;
		gid_t  *  grouplist;
		condor_errno_t terrno;

		assert( syscall_sock->code(gidsetsize) );
		dprintf( D_SYSCALLS, "	gidsetsize = %d\n", gidsetsize );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  ) CONDOR_NotSupported( CONDOR_setgroups );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_plock:
	  {
		int   ret_val;
		int   op;
		condor_errno_t terrno;

		assert( syscall_sock->code(op) );
		dprintf( D_SYSCALLS, "	op = %d\n", op );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  ) CONDOR_NotSupported( CONDOR_plock );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_adjtime:
	  {
		int   ret_val;
		struct timeval *  delta;
		struct timeval *  old_delta;
		condor_errno_t terrno;

		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  ) CONDOR_NotSupported( CONDOR_adjtime );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_sethostid:
	  {
		int   ret_val;
		int   host_id;
		condor_errno_t terrno;

		assert( syscall_sock->code(host_id) );
		dprintf( D_SYSCALLS, "	host_id = %d\n", host_id );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  ) CONDOR_NotSupported( CONDOR_sethostid );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_async_daemon:
	  {
		int   ret_val;
		condor_errno_t terrno;

		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  ) CONDOR_NotSupported( CONDOR_async_daemon );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_getfh:
	  {
		int   ret_val;
		int   fd;
		fhandle_t *  fhp;
		int   exp_fd;
		condor_errno_t terrno;

		assert( syscall_sock->code(fd) );
		dprintf( D_SYSCALLS, "	fd = %d\n", fd );
		assert( syscall_sock->code(exp_fd) );
		dprintf( D_SYSCALLS, "	exp_fd = %d\n", exp_fd );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  ) CONDOR_NotSupported( CONDOR_getfh );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_exportfs:
	  {
		int   ret_val;
		int   access;
		int *  record;
		struct exportfsdata *  exdata;
		condor_errno_t terrno;

		assert( syscall_sock->code(access) );
		dprintf( D_SYSCALLS, "	access = %d\n", access );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  ) CONDOR_NotSupported( CONDOR_exportfs );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_mount:
	  {
		int   ret_val;
		char *  spec;
		char *  dir;
		char *  fstyp;
		unsigned long   rwflag;
		void *  data;
		condor_errno_t terrno;

		assert( syscall_sock->code(rwflag) );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  ) CONDOR_NotSupported( CONDOR_mount );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_umount:
	  {
		int   ret_val;
		char *  dir;
		condor_errno_t terrno;

		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  ) CONDOR_NotSupported( CONDOR_umount );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_acct:
	  {
		int   ret_val;
		char *  path;
		condor_errno_t terrno;

		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  ) CONDOR_NotSupported( CONDOR_acct );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_reboot:
	  {
		int   ret_val;
		int   magic;
		int   magic_too;
		int   flag;
		condor_errno_t terrno;

		assert( syscall_sock->code(magic) );
		dprintf( D_SYSCALLS, "	magic = %d\n", magic );
		assert( syscall_sock->code(magic_too) );
		dprintf( D_SYSCALLS, "	magic_too = %d\n", magic_too );
		assert( syscall_sock->code(flag) );
		dprintf( D_SYSCALLS, "	flag = %d\n", flag );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  ) CONDOR_NotSupported( CONDOR_reboot );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_sethostname:
	  {
		int   ret_val;
		char *  address;
		size_t   address_len;
		condor_errno_t terrno;

		assert( syscall_sock->code(address_len) );
		dprintf( D_SYSCALLS, "	address_len = %lu\n", address_len );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  ) CONDOR_NotSupported( CONDOR_sethostname );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_settimeofday:
	  {
		int   ret_val;
		struct timeval *  tp;
		struct timezone *  tzp;
		condor_errno_t terrno;

		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  ) CONDOR_NotSupported( CONDOR_settimeofday );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_setdomainname:
	  {
		int   ret_val;
		char *  name;
		size_t   namelen;
		condor_errno_t terrno;

		assert( syscall_sock->code(namelen) );
		dprintf( D_SYSCALLS, "	namelen = %lu\n", namelen );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  ) CONDOR_NotSupported( CONDOR_setdomainname );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_swapon:
	  {
		int   ret_val;
		char *  path;
		int   swapflags;
		condor_errno_t terrno;

		assert( syscall_sock->code(swapflags) );
		dprintf( D_SYSCALLS, "	swapflags = %d\n", swapflags );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  ) CONDOR_NotSupported( CONDOR_swapon );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_swapoff:
	  {
		int   ret_val;
		char *  path;
		condor_errno_t terrno;

		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  ) CONDOR_NotSupported( CONDOR_swapoff );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_bdflush:
	  {
		int   ret_val;
		int   func;
		long   data;
		condor_errno_t terrno;

		assert( syscall_sock->code(func) );
		dprintf( D_SYSCALLS, "	func = %d\n", func );
		assert( syscall_sock->code(data) );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  ) CONDOR_NotSupported( CONDOR_bdflush );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_create_module:
	  {
		caddr_t   ret_val;
		char *  name;
		size_t   size;
		condor_errno_t terrno;

		assert( syscall_sock->code(size) );
		dprintf( D_SYSCALLS, "	size = %lu\n", size );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (caddr_t  ) CONDOR_NotSupported( CONDOR_create_module );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %p, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val == NULL  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_delete_module:
	  {
		caddr_t   ret_val;
		char *  name;
		condor_errno_t terrno;

		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (caddr_t  ) CONDOR_NotSupported( CONDOR_delete_module );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %p, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val == NULL  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_init_module:
	  {
		int   ret_val;
		char *  name;
		struct module *  image;
		condor_errno_t terrno;

		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  ) CONDOR_NotSupported( CONDOR_init_module );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_iopl:
	  {
		int   ret_val;
		int   level;
		condor_errno_t terrno;

		assert( syscall_sock->code(level) );
		dprintf( D_SYSCALLS, "	level = %d\n", level );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  ) CONDOR_NotSupported( CONDOR_iopl );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_idle:
	  {
		condor_errno_t terrno;

		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		(void) CONDOR_NotSupported( CONDOR_idle );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = (void), errno = N/A\n" );

		return 0;
	}

	case CONDOR_ioperm:
	  {
		int   ret_val;
		unsigned long   from;
		unsigned long   num;
		int   turn_on;
		condor_errno_t terrno;

		assert( syscall_sock->code(from) );
		assert( syscall_sock->code(num) );
		assert( syscall_sock->code(turn_on) );
		dprintf( D_SYSCALLS, "	turn_on = %d\n", turn_on );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  ) CONDOR_NotSupported( CONDOR_ioperm );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_fork:
	  {
		pid_t   ret_val;
		condor_errno_t terrno;

		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (pid_t  ) CONDOR_NotSupported( CONDOR_fork );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_vfork:
	  {
		pid_t   ret_val;
		condor_errno_t terrno;

		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (pid_t  ) CONDOR_NotSupported( CONDOR_vfork );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_execve:
	  {
		int   ret_val;
		char *  path;
		char * *  argv;
		char * *  envp;
		condor_errno_t terrno;

		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  ) CONDOR_NotSupported( CONDOR_execve );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_execv:
	  {
		int   ret_val;
		char *  path;
		char * *  argv;
		condor_errno_t terrno;

		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  ) CONDOR_NotSupported( CONDOR_execv );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_exec_with_loader:
	  {
		int   ret_val;
		int   flags;
		char *  loader;
		char *  file;
		char * *  argv;
		char * *  envp;
		condor_errno_t terrno;

		assert( syscall_sock->code(flags) );
		dprintf( D_SYSCALLS, "	flags = %d\n", flags );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  ) CONDOR_NotSupported( CONDOR_exec_with_loader );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_execvp:
	  {
		int   ret_val;
		char *  file;
		char * *  argv;
		condor_errno_t terrno;

		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  ) CONDOR_NotSupported( CONDOR_execvp );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_wait4:
	  {
		pid_t   ret_val;
		pid_t   pid;
		void *  status;
		int   options;
		struct rusage *  ru;
		condor_errno_t terrno;

		assert( syscall_sock->code(pid) );
		assert( syscall_sock->code(options) );
		dprintf( D_SYSCALLS, "	options = %d\n", options );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (pid_t  ) CONDOR_NotSupported( CONDOR_wait4 );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_wait:
	  {
		pid_t   ret_val;
		int *  status;
		condor_errno_t terrno;

		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (pid_t  ) CONDOR_NotSupported( CONDOR_wait );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_waitpid:
	  {
		pid_t   ret_val;
		pid_t   pid;
		int *  status;
		int   options;
		condor_errno_t terrno;

		assert( syscall_sock->code(pid) );
		assert( syscall_sock->code(options) );
		dprintf( D_SYSCALLS, "	options = %d\n", options );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (pid_t  ) CONDOR_NotSupported( CONDOR_waitpid );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_ptrace:
	  {
		int   ret_val;
		long   req;
		long int   process;
		unsigned long *  address;
		unsigned long   data;
		condor_errno_t terrno;

		assert( syscall_sock->code(req) );
		assert( syscall_sock->code(process) );
		assert( syscall_sock->code(data) );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  ) CONDOR_NotSupported( CONDOR_ptrace );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_setpgid:
	  {
		int   ret_val;
		pid_t   process_id;
		pid_t   gid;
		condor_errno_t terrno;

		assert( syscall_sock->code(process_id) );
		assert( syscall_sock->code(gid) );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  ) CONDOR_NotSupported( CONDOR_setpgid );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_setsid:
	  {
		pid_t   ret_val;
		condor_errno_t terrno;

		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (pid_t  ) CONDOR_NotSupported( CONDOR_setsid );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_setpgrp:
	  {
		pid_t   ret_val;
		condor_errno_t terrno;

		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (pid_t  ) CONDOR_NotSupported( CONDOR_setpgrp );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_profil:
	  {
		int   ret_val;
		char *  buf;
		int   buf_size;
		int   offset;
		int   scale;
		condor_errno_t terrno;

		assert( syscall_sock->code(buf_size) );
		dprintf( D_SYSCALLS, "	buf_size = %d\n", buf_size );
		assert( syscall_sock->code(offset) );
		dprintf( D_SYSCALLS, "	offset = %d\n", offset );
		assert( syscall_sock->code(scale) );
		dprintf( D_SYSCALLS, "	scale = %d\n", scale );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  ) CONDOR_NotSupported( CONDOR_profil );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_quotactl:
	  {
		int   ret_val;
		char *  path;
		int   cmd;
		int   id;
		char *  addr;
		condor_errno_t terrno;

		assert( syscall_sock->code(cmd) );
		dprintf( D_SYSCALLS, "	cmd = %d\n", cmd );
		assert( syscall_sock->code(id) );
		dprintf( D_SYSCALLS, "	id = %d\n", id );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  ) CONDOR_NotSupported( CONDOR_quotactl );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_getsysinfo:
	  {
		int   ret_val;
		unsigned long   op;
		char *  buf;
		unsigned long   nbytes;
		int *  start;
		char *  arg;
		condor_errno_t terrno;

		assert( syscall_sock->code(op) );
		assert( syscall_sock->code(nbytes) );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  ) CONDOR_NotSupported( CONDOR_getsysinfo );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_setitimer:
	  {
		int   ret_val;
		int   which;
		struct itimerval *  value;
		struct itimerval *  oval;
		condor_errno_t terrno;

		assert( syscall_sock->code(which) );
		dprintf( D_SYSCALLS, "	which = %d\n", which );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  ) CONDOR_NotSupported( CONDOR_setitimer );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_getitimer:
	  {
		int   ret_val;
		int   which;
		struct itimerval *  value;
		condor_errno_t terrno;

		assert( syscall_sock->code(which) );
		dprintf( D_SYSCALLS, "	which = %d\n", which );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  ) CONDOR_NotSupported( CONDOR_getitimer );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_msgctl:
	  {
		int   ret_val;
		int   msgid;
		int   cmd;
		struct msqid_ds *  buf;
		condor_errno_t terrno;

		assert( syscall_sock->code(msgid) );
		dprintf( D_SYSCALLS, "	msgid = %d\n", msgid );
		assert( syscall_sock->code(cmd) );
		dprintf( D_SYSCALLS, "	cmd = %d\n", cmd );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  ) CONDOR_NotSupported( CONDOR_msgctl );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_msgget:
	  {
		int   ret_val;
		key_t   key;
		int   msgflg;
		condor_errno_t terrno;

		assert( syscall_sock->code(key) );
		assert( syscall_sock->code(msgflg) );
		dprintf( D_SYSCALLS, "	msgflg = %d\n", msgflg );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  ) CONDOR_NotSupported( CONDOR_msgget );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_msgrcv:
	  {
		int   ret_val;
		int   qid;
		void *  msgp;
		size_t   msgsz;
		long   msgtyp;
		int   msgflg;
		condor_errno_t terrno;

		assert( syscall_sock->code(qid) );
		dprintf( D_SYSCALLS, "	qid = %d\n", qid );
		assert( syscall_sock->code(msgsz) );
		dprintf( D_SYSCALLS, "	msgsz = %lu\n", msgsz );
		assert( syscall_sock->code(msgtyp) );
		assert( syscall_sock->code(msgflg) );
		dprintf( D_SYSCALLS, "	msgflg = %d\n", msgflg );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  ) CONDOR_NotSupported( CONDOR_msgrcv );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_msgsnd:
	  {
		int   ret_val;
		int   msqid;
		void *  msgp;
		size_t   msgsz;
		int   msgflg;
		condor_errno_t terrno;

		assert( syscall_sock->code(msqid) );
		dprintf( D_SYSCALLS, "	msqid = %d\n", msqid );
		assert( syscall_sock->code(msgsz) );
		dprintf( D_SYSCALLS, "	msgsz = %lu\n", msgsz );
		assert( syscall_sock->code(msgflg) );
		dprintf( D_SYSCALLS, "	msgflg = %d\n", msgflg );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  ) CONDOR_NotSupported( CONDOR_msgsnd );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_semctl:
	  {
		int   ret_val;
		int   semid;
		int   semnum;
		int   cmd;
		void *  arg;
		condor_errno_t terrno;

		assert( syscall_sock->code(semid) );
		dprintf( D_SYSCALLS, "	semid = %d\n", semid );
		assert( syscall_sock->code(semnum) );
		dprintf( D_SYSCALLS, "	semnum = %d\n", semnum );
		assert( syscall_sock->code(cmd) );
		dprintf( D_SYSCALLS, "	cmd = %d\n", cmd );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  ) CONDOR_NotSupported( CONDOR_semctl );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_semget:
	  {
		int   ret_val;
		key_t   key;
		int   nsems;
		int   semflag;
		condor_errno_t terrno;

		assert( syscall_sock->code(key) );
		assert( syscall_sock->code(nsems) );
		dprintf( D_SYSCALLS, "	nsems = %d\n", nsems );
		assert( syscall_sock->code(semflag) );
		dprintf( D_SYSCALLS, "	semflag = %d\n", semflag );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  ) CONDOR_NotSupported( CONDOR_semget );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_semop:
	  {
		int   ret_val;
		int   semid;
		struct sembuf *  sops;
		unsigned int   nsops;
		condor_errno_t terrno;

		assert( syscall_sock->code(semid) );
		dprintf( D_SYSCALLS, "	semid = %d\n", semid );
		assert( syscall_sock->code(nsops) );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  ) CONDOR_NotSupported( CONDOR_semop );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_shmctl:
	  {
		int   ret_val;
		int   shmid;
		int   cmd;
		struct shmid_ds *  buf;
		condor_errno_t terrno;

		assert( syscall_sock->code(shmid) );
		dprintf( D_SYSCALLS, "	shmid = %d\n", shmid );
		assert( syscall_sock->code(cmd) );
		dprintf( D_SYSCALLS, "	cmd = %d\n", cmd );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  ) CONDOR_NotSupported( CONDOR_shmctl );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_shmdt:
	  {
		int   ret_val;
		void *  addr;
		condor_errno_t terrno;

		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  ) CONDOR_NotSupported( CONDOR_shmdt );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_shmget:
	  {
		int   ret_val;
		key_t   key;
		size_t   size;
		int   flags;
		condor_errno_t terrno;

		assert( syscall_sock->code(key) );
		assert( syscall_sock->code(size) );
		dprintf( D_SYSCALLS, "	size = %lu\n", size );
		assert( syscall_sock->code(flags) );
		dprintf( D_SYSCALLS, "	flags = %d\n", flags );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (int  ) CONDOR_NotSupported( CONDOR_shmget );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %d, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val < 0  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

	case CONDOR_shmat:
	  {
		void *  ret_val;
		int   shmid;
		void *  shmaddr;
		int   shmflg;
		condor_errno_t terrno;

		assert( syscall_sock->code(shmid) );
		dprintf( D_SYSCALLS, "	shmid = %d\n", shmid );
		assert( syscall_sock->code(shmflg) );
		dprintf( D_SYSCALLS, "	shmflg = %d\n", shmflg );
		assert( syscall_sock->end_of_message() );;

		errno = (condor_errno_t)0;
		ret_val = (void * ) CONDOR_NotSupported( CONDOR_shmat );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\tret_val = %p, errno = %d\n", ret_val, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(ret_val) );
		if(  ret_val == NULL  ) {
			assert( syscall_sock->code(terrno) );
		}
		assert( syscall_sock->end_of_message() );;
		return 0;
	}

/***************************************************************
 *
 * Copyright (C) 1990-2007, Condor Team, Computer Sciences Department,
 * University of Wisconsin-Madison, WI.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you
 * may not use this file except in compliance with the License.  You may
 * obtain a copy of the License at
 * 
 *    http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ***************************************************************/

	default:
	{
		condor_errno_t terrno;

		/* consume arguments */
		syscall_sock->end_of_message();

		errno = 0;
		rval = CONDOR_NotSupported( condor_sysnum );
		terrno = (condor_errno_t)errno;
		dprintf( D_SYSCALLS, "\trval = %d, errno = %d\n", rval, (int)terrno );

		syscall_sock->encode();
		assert( syscall_sock->code(rval) );
		assert( syscall_sock->code(terrno) );
		assert( syscall_sock->end_of_message() );;
		return 0;
	}


}	/* End of switch on system call number */
}	/* End of do_REMOTE_syscall() procedure */

int
CONDOR_NotSupported( int condor_sysnum,  ... )
{
	job_report_store_error( "system call not supported by Condor",
	                      _condor_syscall_name(condor_sysnum) );

	errno = ENOSYS;
	return -1;
}

int
CONDOR_Ignored( int condor_sysnum,  ... )
{
	dprintf(
		D_SYSCALLS,
		"%s - system call not supported by Condor - IGNORED!",
		_condor_syscall_name(condor_sysnum)
	);
	return 0;
}
